''' Eventscripts eXtended:
    By your-name-here
    Special thanks to Mattie :D '''
    

''' 
Your-name-here NOTE:
    I [STRONGLY] urge scripters to use my classes because they provide additional
    error checking that is would be too tedious to do plugin-side. Remember that
    if you are calling functions directly, the chance of crashes are higher than
    using the esx built in methods and classes.
'''

#================================================================
# Imports 
#================================================================
import es
from esx_C import * # Import all functions from the esx_C module

#================================================================
# Public Variables
#================================================================
__version__ = '1.0.0a'
es.ServerVar('esx_version', __version__).makepublic()

#================================================================
# Globals
#================================================================

class NullPointerException(Exception):
    pass

class InvalidEntityException(Exception):
    pass


#================================================================
# Functionality
#================================================================

''' CBaseEntity Class:
        Almost every entity descends from this
'''
class CBaseEntity():

    '''
        There are three different versions of __init__.
        This one uses an entity index
    '''
    def __init__(self, entity_name):
        
        # Get the pointer to this entity
        self.ptr = createEntityByName(entity_name)
        
        if self.ptr == None:
            raise InvalidEntityException("Entity %s is not valid!" % entity_name)
        
        # User created entities are usually deletable
        self.deletable = True
    
    '''
        This version takes a POINTER to an entity as the first
        argument and a boolean as the second. When this class is
        destroyed, if bDeletable is True, this entity will be removed
        from the world using UTIL_Remove. If it's false, the entity will
        not be removed.
    '''
    def __init__(self, entity_ptr, bDeletable=False):
        
        # Store the boolean
        self.deletable = bDeletable
        
        # Store the entity pointer
        if entity_ptr == None:
            raise NullPointerException("Null pointer found!")
        else:
            self.ptr = entity_ptr
    
    '''
        Deletes the entity if self.deletable is true
    '''
    
    def __del__(self):
        
        # Delete the entity if we are supposed to
        if self.deletable == True:
            removeEntity(self.ptr)
    
    '''
        This method sets the keyvalues of any entity that descends from CBaseEntity.
        
        Parameters:
            type - Type of keyvalue that you'll be setting (can be string or float presently)
            keyvalue - Name of the keyvalue to change
            value - The value you want to change the keyvalue to
        
    '''
    def setKeyvalues(self, type, keyvalue, value):
        
        if type == 'string':
            setKvString(self.ptr, keyvalue, value)
        
        if type == 'float':
            setKvFloat(self.ptr, keyvalue, value)
          

'''
    This class represents a Counter-Strike source player.
    It descends from CBaseEntity, and consequently,
    it has every method CBaseEntity has.
    
    NOTE: You can't call removeEntity on CCSPlayer instances.
          The game will crash!
'''
class CCSPlayer(CBaseEntity):

    def __init__(self, userid):
        
        # Set the userid
        self.userid = userid
        
        # Get the entity of the userid
        self.ptr = getEntityOfUserid(self.userid)
        
        # Check to see if it's valid
        if self.ptr == None:
            raise NullPointerException("Userid %i or entity pointer is invalid!" % userid)
        
        # Deletable is false
        self.deletable = False

    '''
        Respawns the player
    '''
    def respawn(self):
        playerRespawn(self.ptr)
    
    '''
        Changes a player's team.
        Parameters:
            iTeamIndex - Index of the team to change the player to.
    '''
    def changeTeam(self, iTeamIndex):
        playerChangeTeam(self.ptr, iTeamIndex)

    '''
        Returns a pointer to a player's weapon
        if they own a weapon of weapon_type.
    '''
    def ownsWeapon(self, weapon_type):
        return playerOwnsWeapon(self.ptr, weapon_type)

    '''
        Strips a player of type weapon_type if they have it.
        Returns True if successful.
    '''
    def stripWeapon(self, weapon_type):
        
        # Get the weapon pointer
        weapon_ptr = self.ownsWeapon(weapon_type)
        
        if weapon_ptr == None:
            return False
        
        # Otherwise strip the entity
        removeEntity(weapon_ptr)

    '''
        Sends a message to a player.
    '''
    def hintMessage(self, message):
        playerHintMessage(self.ptr, message)
    

'''
    This is a very special class called CCSGameRules.
    It controls various gameplay aspects.
'''
class CCSGameRules(CBaseEntity):

    def __init__(self):
        
        # Get the pointer to CCSGamerules
        self.ptr = getGameRules()
        
        if self.ptr == None:
            raise NullPointerException("Gamerules pointer was null!")
        
        # You can't delete this
        self.deletable = False
    
    '''
        This method restarts a round in CSS.
    '''
    def roundRestart(self):
        rulesRestartRound(self.ptr)














